// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <iostream>

#include <anna/core/util/CommandLine.hpp>
#include <anna/core/DataBlock.hpp>
#include <anna/core/tracing/TraceWriter.hpp>
#include <anna/core/functions.hpp>

#include <anna/app/Application.hpp>

#include <anna/comm/comm.hpp>

using namespace anna;

class MyCodec : public Codec {
public:
   MyCodec () : Codec (0xaa) ,
      a_dataBlock (true)
   {
      attach ("string", a_string);
      attach ("integer", a_integer);
      attach ("block", a_dataBlock);
      attach ("string", a_string2);
      attach ("float", a_float);
      attach ("double", a_double);
   }
   
   void set (const std::string& value) throw () { a_string = a_string2 = value; a_string2 += value; }
   void set (const int value) throw () { a_integer = value; }
   void set (const anna::DataBlock& value) throw () { a_dataBlock = value; }
   void set (const float value) throw () { a_float = value; }
   void set (const double value) throw () { a_double = value; }
   
   const std::string& getString () const throw () { return a_string; }
   const int getInteger () const throw () { return a_integer; }
   const anna::DataBlock& getDataBlock () const throw (anna::RuntimeException) { return a_dataBlock; }
   const std::string& getString2 () const throw () { return a_string2; }
   float getFloat () const throw () { return a_float; }
   float getDouble () const throw () { return a_double; }
   
private:
   std::string a_string;
   int a_integer;
   anna::DataBlock a_dataBlock;
   std::string a_string2;
   float a_float;
   double a_double;
};

class Test : public anna::app::Application {
public:
   Test ();
   
private:
   MyCodec a_input;
   MyCodec a_output;

   void run () throw (anna::RuntimeException);      
};

using namespace std;
using namespace anna;

int main (int argc, const char** argv)
{
   CommandLine& commandLine (CommandLine::instantiate ());
   Test test;
   TraceWriter* traceWriter = new TraceWriter;

   try {
      commandLine.initialize (argv, argc);
      commandLine.verify ();

      Logger::initialize ("testfunctions", traceWriter);
      Logger::setLevel (Logger::Debug); 
 
      test.start ();
   }
   catch (Exception& ex) {
      cout << ex.asString () << endl;
   }
   
   delete traceWriter;
   
   return 0;
}

Test::Test () : 
   anna::app::Application ("testfunctions", "Comprobacion de la clase comm::Codec", "1.0.0") 
{
   CommandLine& commandLine (CommandLine::instantiate ());
      
   commandLine.add ("string", CommandLine::Argument::Mandatory, "Cadena a enviar");
   commandLine.add ("i", CommandLine::Argument::Mandatory, "Enterno a enviar");
}

void Test::run () 
   throw (RuntimeException)
{
   CommandLine& cm (CommandLine::instantiate ());
   
   const char* string = cm.getValue ("string");

   float ff;
   double dd;
   
   a_input.set (string);
   a_input.set (cm.getIntegerValue ("i"));
   a_input.set (ff = 1.12345);
   a_input.set (dd = 2.12345);
   
   DataBlock dataBlock (true);
   
   for (int i = strlen (string) - 1; i > 0; i --)
      dataBlock += string [i];
      
   a_input.set (dataBlock);

   const DataBlock& result (a_input.code ());

   a_output.decode (result);
   
   cout << "String: " << a_output.getString () << endl;
   cout << "Integer: " << a_output.getInteger () << endl;
   cout << "Data block: " << anna::functions::asString (a_output.getDataBlock ()) << endl;
   cout << "String2: " << a_output.getString2 () << endl << endl;
   cout << "Float: " << a_output.getFloat () << endl << endl;
   cout << "Double: " << a_output.getDouble () << endl << endl;

   /*
    * Para comprobar que el modo de codificar los bytes son igual que en la versin de Java
    */
   char buffer [sizeof (S64)];

   short ss = 23456;
   comm::functions::codeShort (buffer, ss);
   dataBlock.clear ();
   dataBlock.append (buffer, sizeof (ss));
   cout << anna::functions::asString ("Short: %d - 0x%x", ss, ss);
   cout << anna::functions::asString (dataBlock) << endl << endl;
   ss = comm::functions::decodeShort (buffer);
   cout << anna::functions::asString ("Short2: %d - 0x%x\n\n", ss, ss);

   int xx = 123456;
   comm::functions::codeInteger (buffer, xx);
   dataBlock.clear ();
   dataBlock.append (buffer, sizeof (xx));
   cout << anna::functions::asString ("Integer: %d - 0x%x", xx, xx);
   cout << anna::functions::asString (dataBlock) << endl << endl;
   xx = comm::functions::decodeInteger (buffer);
   cout << anna::functions::asString ("Integer2: %d - 0x%x\n\n", xx, xx);

//   S64 ll = 31604938272LL;
   S64 ll = 98765432101234LL;
   comm::functions::codeInteger64 (buffer, ll);
   dataBlock.clear ();
   dataBlock.append (buffer, sizeof (ll));
   cout << anna::functions::asString ("Integer64: %lld - 0x%llx", ll, ll);
   cout << anna::functions::asString (dataBlock) << endl << endl;
   ll = comm::functions::decodeInteger64 (buffer);
   cout << anna::functions::asString ("Integer64-2: %lld - 0x%llx\n\n", ll, ll);
  
   cout << "Sizeof (float/int): " << sizeof (float) << "/" << sizeof (int) << endl;

   ff = -123.3232;
   int ii;
   anna_memcpy (&ii, &ff, sizeof (ff));
   comm::functions::codeInteger (buffer, ii);
   cout << "Float I: " << ff << " (" << anna::functions::asHexString (ii) << ") " << endl;

   float ff2;
   ii = comm::functions::decodeInteger (buffer);
   anna_memcpy (&ff2, &ii, sizeof (ff));
   cout << "Float O: " << ff2 << " (" << anna::functions::asHexString (ii) << ") " << endl;

   dd = -123123123.3232;
   S64 ii64;
   anna_memcpy (&ii64, &dd, sizeof (dd));
   comm::functions::codeInteger64 (buffer, ii64);
   cout << "Double I: " << dd << " (" << anna::functions::asHexString (ii64) << ") " << endl;

   double dd2;
   ii64 = comm::functions::decodeInteger64 (buffer);
   anna_memcpy (&dd2, &ii64, sizeof (dd));
   cout << "Double O: " << dd2 << " (" << anna::functions::asHexString (ii64) << ") " << endl;
}

