// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


/*
   Establece un manejador externo para controlar el teclado, recoge los parametros de la operacion
   por este y envia la peticion al servidor que devolvera el resultado de aplicar la operacion 
   sobre los parametros recogidos.

   Este proceso, a diferencia de kclient, abre una nueva conexion por cada peticion que tiene que lanzar.

   El cliente de esta aplicacion es: server.p => Transporte: comm::Transport.
*/
#include <iostream>

#include <string.h>

#include <anna/core/core.hpp>

#include <anna/comm/Application.hpp>
#include <anna/comm/Communicator.hpp>
#include <anna/comm/INetAddress.hpp>
#include <anna/comm/ClientSocket.hpp>
#include <anna/comm/Network.hpp>
#include <anna/comm/Device.hpp>

#include <anna/test/Menu.hpp>
#include <anna/test/Request.hpp>
#include <anna/test/Response.hpp>

class MyCommunicator : public comm::Communicator {
public:   
   class ClientSocketAllocator {
   public:
      static comm::INetAddress* st_inetAddress;      
      static comm::ClientSocket* create ()  throw () { return new comm::ClientSocket (*st_inetAddress); }
      static void destroy (comm::ClientSocket* clientSocket) throw () { delete clientSocket; }
   };   
   
private:
   using comm::Communicator::eventBreakConnection;

   test::Response a_response;   
   test::Request a_request;
   comm::INetAddress a_inetAddress;
   Recycler <comm::ClientSocket, ClientSocketAllocator> a_clientSockets;

   void do_initialize () throw (RuntimeException);
   
   void eventReceiveMessage (comm::ClientSocket&, const comm::Message&) throw (RuntimeException);
   void eventBreakConnection (const comm::ClientSocket&) throw ();
   void eventUser (const char* id, const void* context) throw ();   
};

class KXClient : public anna::comm::Application {
public:
   KXClient ();
      
   const test::Menu& getMenu () const throw () { return a_menu; }
   
private:
   MyCommunicator a_communicator;
   test::Menu a_menu;

   void initialize () throw (RuntimeException);
   void run () throw (RuntimeException);    
};

using namespace std;
using namespace test;

comm::INetAddress* MyCommunicator::ClientSocketAllocator::st_inetAddress = NULL;

int main (int argc, const char** argv)
{
   CommandLine& commandLine (CommandLine::instantiate ());
   KXClient app;
   
   try {
      commandLine.initialize (argv, argc);
      commandLine.verify ();

      Logger::setLevel (Logger::Debug); 
      Logger::initialize ("kclient", new TraceWriter ("file.trace", 4096000));
 
      app.start ();
   }
   catch (Exception& ex) {
      cout << ex.asString () << endl;
   }
   
   return 0;
}

KXClient::KXClient () : 
   Application ("kclient", "KXClient", "1.0.0"),
   a_menu (&a_communicator)
{
   CommandLine& commandLine (CommandLine::instantiate ());
      
   commandLine.add ("p", CommandLine::Argument::Mandatory, "Puerto en el que el servidor atiende respuestas.");
   commandLine.add ("a", CommandLine::Argument::Mandatory, "Direccion IP Puerto en el que el servidor atiende respuestas.");
}

void KXClient::initialize () 
   throw (RuntimeException)
{
   a_communicator.attach (&a_menu);
}

void KXClient::run ()
   throw (RuntimeException)
{   
   a_menu.paint ();
   a_communicator.accept ();
}

//--------------------------------------------------------------------------------------------
// Crea la direccion a la que se conectaran los ClientSocket para enviar las peticiones.
//--------------------------------------------------------------------------------------------
void MyCommunicator::do_initialize () 
   throw (RuntimeException)
{
   CommandLine& cl (CommandLine::instantiate ());    

   using namespace anna::comm;

   Network& network = Network::instantiate ();
   
   Device* device = network.find (Device::asAddress (cl.getValue ("a")));
   
   a_inetAddress.setAddress (device);
   a_inetAddress.setPort (cl.getIntegerValue ("p"));
   
   ClientSocketAllocator::st_inetAddress = &a_inetAddress;
}

void MyCommunicator::eventReceiveMessage (comm::ClientSocket&, const comm::Message& message)
   throw (RuntimeException)
{
   a_response.decode (message.getBody ());
   
   cout << endl << "Resultado de la peticion: " << a_response.x << (char) a_response.op << a_response.y  << " = " << a_response.result << endl << endl;
   
   static_cast <KXClient&> (anna::comm::functions::getApp ()).getMenu ().paint ();
}

//-----------------------------------------------------------------------------------------
// Cuando el servidor remoto cierra el socket => debemos liberar este extremo para poder
// reutilizar la instancia (no la conexion).
//-----------------------------------------------------------------------------------------
void MyCommunicator::eventBreakConnection (const comm::ClientSocket& clientSocket) 
   throw ()
{
   a_clientSockets.release (&clientSocket);
}

//-----------------------------------------------------------------------------------------
// Cuando el Menu tiene disponibles todos los datos necesarios para la peticin se lo
// notifica al comunicador mediante un evento de usuario.
//-----------------------------------------------------------------------------------------
void MyCommunicator::eventUser (const char* id, const void* context) 
   throw ()
{
   LOGMETHOD (TraceMethod tm ("MyCommunicator", "eventUser", ANNA_FILE_LOCATION));

   if (anna_strcmp (id, Menu::EventData) == 0) {
      const Menu::Data* data (reinterpret_cast <const Menu::Data*>  (context));
      
      a_request.op = data->a_operation;
      a_request.x = data->a_op1;
      a_request.y = data->a_op2;
      
      comm::ClientSocket* clientSocket = a_clientSockets.create ();
            
      try {
         attach (clientSocket);
         clientSocket->send (a_request);
      }
      catch (RuntimeException& ex) {
         a_clientSockets.release (clientSocket);
         ex.trace ();
      }
   } 
}
