#!/bin/bash

#############
# VARIABLES #
#############
SCR_DIR=`readlink -f $0 | xargs dirname`
PROJECT_ROOT=$(readlink -f $SCR_DIR/../../..)

VARIANT=${1:-Release}

#############
# FUNCTIONS #
#############
_exit () {
  local msg="$1"
  local rc=$2
  [ -z "$rc" ] && rc=1

  # Exit message:
  [ -n "$msg" ] && echo -e "\n${msg}\n"

  exit $rc
}

#############
# EXECUTION #
#############

echo
echo "------------------------------------------------------"
echo "         ADML with HTTP support installation          "
echo "------------------------------------------------------"
echo
[ ! -d $PROJECT_ROOT/build/${VARIANT} ] && _exit "Cannot locate neither '${VARIANT}' variant. Build the project !"
echo "Variant: ${VARIANT}"
echo
build_type_letter=$(echo $VARIANT | cut -c1 | tr '[:upper:]' '[:lower:]')

version__dflt=v"`date +'%y'`.`date +'%m'`.`date +'%d'`$build_type_letter"
INSTALL__dflt=$HOME/3rdParty/anna-adml-http-builds/v"`date +'%y'`.`date +'%m'`.`date +'%d'`$build_type_letter"

echo "Choose the target path for installation [$INSTALL__dflt]:"
echo " (enter a non-existent directory)"
read INSTALL
[ -z "$INSTALL" ] && INSTALL=$INSTALL__dflt
INSTALL=`readlink -m $INSTALL`
[ -d $INSTALL ] && _exit "The target installation directory ($INSTALL) already exists ! (if you want to reinstall, remove it first)"

echo
echo "Stage 1: Deploying resources ......................"
echo
mkdir -p $INSTALL
cp -rL $SCR_DIR/deployments/adml/* $INSTALL

# VARIABLES ######################################
# Sources:
LDIR=${PROJECT_ROOT}/example/diameter/launcher
BIN_DIR=${PROJECT_ROOT}/build/$VARIANT/bin
LIB_DIR=${PROJECT_ROOT}/build/$VARIANT/lib

# Targets:
ADML=${INSTALL}
DTDs=${ADML}/DTDs
DYNLIBS=${ADML}/dynlibs
##################################################

echo
echo "Variant: $VARIANT"
echo

# Empty directories:
mkdir -p ${ADML}/counters
mkdir -p ${ADML}/test-reports

# Scripts:
cp ${PROJECT_ROOT}/example/diameter/launcher/resources/scripts/operation.sh ${ADML}

# Templates:
mkdir $DTDs
cp ${PROJECT_ROOT}/include/anna/diameter/codec/message.dtd ${DTDs}
cp ${PROJECT_ROOT}/include/anna/diameter/stack/dictionary.dtd ${DTDs}
cp ${PROJECT_ROOT}/example/diameter/launcher/resources/services_examples/services.dtd ${DTDs}

# Main Launcher and dynamic libraries:
cp ${BIN_DIR}/anna_diameter_launcher ${ADML}/ADML
cp -r $LIB_DIR/dynamic/launcher ${DYNLIBS}
cp $LIB_DIR/libanna_testing_shared.so ${DYNLIBS}

# Get stuff from leaf directories:
cd $LIB_DIR/dynamic/launcher
leafs=( $(find . -type d -links 2) )
cd - >/dev/null

cd ${PROJECT_ROOT}/dynamic/launcher
for dir in ${leafs[@]}
do
    cp $dir/*.xml ${DYNLIBS}/$dir 2>/dev/null
    cp $dir/dynamic.suffix ${DYNLIBS}/$dir 2>/dev/null
    cp -r $dir/services ${DYNLIBS}/$dir 2>/dev/null
done
cd - >/dev/null

# ADML dynamic libs selection script:
cp ${LDIR}/resources/scripts/select_dynlib.sh ${DYNLIBS}/select.sh

# Default dynamic library:
cd ${DYNLIBS}
ln -sf default/libanna_launcher_procedure_default_shared.so
cd - >/dev/null

# ADML Agent shall use http server:
httpOpt="--httpServerShared --httpServer \`grep -v ^# .httpServer\`"
echo "${httpOpt}" >> ${ADML}/args.ft
echo "${httpOpt}" >> ${ADML}/args.st
echo "localhost:8000" > ${ADML}/.httpServer

# Help
cp ${PROJECT_ROOT}/example/diameter/launcher/resources/HELP.md ${ADML}

# Simple services which only loads the diameter base dictionary:
cat << EOF > ${ADML}/services.xml
<services>
  <stack id="0" dictionary="stacks/diameter_base.0.xml" fixMode="Never"/>
</services>
EOF

# Remove AOTS actions:
rm ${ADML}/ACTIONS.md

_exit "Done!" 0

