// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_app_functions_hpp
#define anna_app_functions_hpp

#include <anna/core/functions.hpp>

#include <anna/app/Application.hpp>

namespace anna {

namespace app {

class Application;

/**
   functions - Metodos y variables
*/
struct functions : public anna::functions {
  /**
     Devuelve la referencia de la instancia de nuestra aplicacion
  */
  static Application& getApp() throw(RuntimeException);


  /**
     Pattern to obtain a multi named application component instance easily.
     Parameters are usually replaced by the macro C <b>FILE_LOCATION</b>.

     \param className Application component class name
     \param fromFile File which called the method
     \param fromLine Line number within the file from where the method is called.

     \return Application component instance for the class provided at the pattern
     \see Component
  */
  template <typename T> static T* componentByName(const char *className, const char* fromFile, const int fromLine)
  throw(anna::RuntimeException) {
    T* result = static_cast <T*>(functions::getApp().find(className));

    if(result == NULL) {
      std::string msg(className);
      msg += " | Componente no registrado";
      throw RuntimeException(msg, fromFile, fromLine);
    }

    return result;
  }
};

  /**
     Pattern to obtain a single named application component instance easily.
     Parameters are usually replaced by the macro C <b>FILE_LOCATION</b>.

     \param fromFile File which called the method
     \param fromLine Line number within the file from where the method is called.

     \return Application component instance for the class provided at the pattern
     \warning T class must implement a method in the form:
     \code
         static const char* getClassName () throw ();
     \endcode
     \see Component
  */
  template <typename T> static T* component(const char* fromFile, const int fromLine)
  throw(RuntimeException) {
    return functions::componentByName<T> (T::getClassName(), fromFile, fromLine);
  }

}
}

#endif


