// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_diameter_helpers_tme_codectypes_ISDNNumber_hpp
#define anna_diameter_helpers_tme_codectypes_ISDNNumber_hpp


// Local
#include <anna/diameter/codec/basetypes/OctetString.hpp>

#include <anna/config/defines.hpp> // isup_number_t


namespace anna {

namespace diameter {

namespace codec {
class Avp;
}

namespace helpers {

namespace tme {

namespace codectypes {

/**
* Diameter ISDNNumber container
  Internal format is based in Calling Party Number (Q763)
*/
class ISDNNumber : public codec::basetypes::OctetString {

  isup_number_t a_isupNumber;

  // Only for derived diameter type:
  void updateBasic() throw(anna::RuntimeException);

public:

  // Class-specific ////////////////////////////////////////////////////////////////////////////////////
  //
  /**
  * Gets the Q763 ISUP Number
  *
  * @return Q763 ISUP Number
  */
  const isup_number_t& getIsupNumber() const throw() { return a_isupNumber; }

  /**
  * Sets the Q763 ISUP Number
  *
  * @param isupNumber Q763 ISUP Number
  */
  void setIsupNumber(const isup_number_t& isupNumber) throw() { a_isupNumber = isupNumber; updateBasic(); }
  //
  //////////////////////////////////////////////////////////////////////////////////////////////////////

  // gets

  std::string getFormatName() const throw() { return "ISDNNumber"; }

  // helpers

  std::string asString() throw(anna::RuntimeException) {
    return a_isupNumber.asString(false /* calling party number */);
  }


  // sets

  void decode(const char* buffer, const int size) throw(anna::RuntimeException);


  // exports /////////////////////////////
  using AvpData::getSize;
  using AvpData::code;
  //using OctetString::asPrintableString;
  using AvpData::asDataBlockString;
  //using OctetString::asString;
  using AvpData::asHexString;
  //using OctetString::decode;
  using AvpData::fromPrintableString;
  using AvpData::fromHexString;

};

}
}
}
}
}

#endif

