// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_http_Handler_hpp
#define anna_http_Handler_hpp

#include <anna/core/mt/Mutex.hpp>

#include <anna/comm/Receiver.hpp>

namespace anna {

class DataBlock;

namespace comm {
class ClientSocket;
class Message;
}

namespace http {

class Request;
class Response;

/**
   Manejador de Respuestas y/o peticiones sobre protocolo HTTP.
*/
class Handler : public comm::Receiver {
public:
  /**
     Destructor.
  */
  virtual ~Handler();

  /**
     Trata la informacion contenida en el mensaje recibido. Si el mensaje recibido es un mensaje HTTP
     valido se terminara llamando a uno de los metodos manejadores evRequest o evResponse, dependiendo
     del tipo de mensaje.

     \param clientSocket Socket por el que se recibe la peticion, y por el que podriamos enviar la respuesta
     en caso de ser necesario.
     \param message Mensaje HTTP recibido.
  */
  void apply(comm::ClientSocket& clientSocket, const comm::Message& message) throw(RuntimeException);

protected:
  /**
     Constructor.
     \param name Nombre logico de este manejador.
  */
  Handler(const char* name) : comm::Receiver(name), a_response(NULL) {;}

  /**
     Devuelve una instancia de http::Response que puede ser usada para responser a una peticion.
     \return una instancia de http::Response que puede ser usada para responser a una peticion.
  */
  Response* allocateResponse() throw();

  /**
     Metodo virtual que debemos sobreescribir para tratar las peticiones HTTP.
     \param clientSocket Socket por el que se recibe la peticion, y por el que podriamos enviar la respuesta
     en caso de ser necesario.
     \param request Peticion HTTP a tratar.
  */
  virtual void evRequest(comm::ClientSocket& clientSocket, const Request& request) throw(RuntimeException) = 0;

  /**
     Metodo virtual que debemos sobreescribir para tratar las respuestas HTTP.
     \param clientSocket Socket por el que se recibe la respuesta.
     \param response respuesta HTTP a tratar.
  */
  virtual void evResponse(comm::ClientSocket& clientSocket, const Response& response) throw(RuntimeException) = 0;

private:
  Response* a_response;
};

}
}

#endif

