// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/tracing/Logger.hpp>
#include <anna/core/functions.hpp>
#include <anna/core/mt/Guard.hpp>

#include <anna/xml/Node.hpp>
#include <anna/xml/Attribute.hpp>

#include <anna/comm/Communicator.hpp>
#include <anna/comm/internal/BinderSocket.hpp>

#include <anna/comm/handler/BinderSocket.hpp>

using namespace std;
using namespace anna;

//-----------------------------------------------------------------------
// La apertura de cierre de este Socket se hace desde el comm::Socket.
//-----------------------------------------------------------------------
void comm::handler::BinderSocket::initialize()
throw(RuntimeException) {
  if(a_binderSocket == NULL) {
    string msg(asString());
    msg += " | comm::BinderSocket was not established";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  setfd(a_binderSocket->getfd());
}

//---------------------------------------------------------------------------------------
// Cuando detecta actividad lo unico que hace es invocar al metodo del BinderSocket
// que deberia estar esperando la respuesta para saber el 'fd' del nuevo socket
// compartido que va a crear.
//---------------------------------------------------------------------------------------
void comm::handler::BinderSocket::apply()
throw(RuntimeException) {
  Guard guard(a_binderSocket);
  a_binderSocket->responseBind();
}

void comm::handler::BinderSocket::finalize()
throw() {
  if(a_binderSocket == NULL)
    return;

  Guard guard(a_binderSocket);
  a_binderSocket->close();
  a_binderSocket = NULL;
}

string comm::handler::BinderSocket::asString() const
throw() {
  string result("comm::handler::BinderSocket { ");
  result += comm::Handler::asString();
  result += " | ";
  result += functions::asString(a_binderSocket);
  return result += " }";
}

xml::Node* comm::handler::BinderSocket::asXML(xml::Node* parent) const
throw() {
  xml::Node* result = parent->createChild("comm.handler.BinderSocket");
  comm::Handler::asAttribute(result);

  if(a_binderSocket)
    a_binderSocket->asXML(result);

  return result;
}


