// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/tracing/Logger.hpp>
#include <anna/core/functions.hpp>

#include <anna/xml/Compiler.hpp>

#include <anna/http/Message.hpp>
#include <anna/http/internal/defines.hpp>
#include <anna/http/Header.hpp>

using namespace std;
using namespace anna;

comm::Message* http::Message::setBody(const xml::Node* node)
throw(RuntimeException) {
  comm::Message::setBody(node);
  http::Header* contentType = find(http::Header::Type::ContentType);

  if(contentType == NULL)
    contentType = createHeader(http::Header::Type::ContentType);

  contentType->setValue("text/xml");
  return this;
}

http::Header* http::Message::find(const http::Header::Type::_v type)
throw() {
  http::Header* result = NULL;

  for(header_iterator ii = header_begin(), maxii = header_end(); ii != maxii; ii ++) {
    if(header(ii)->getType() == type) {
      result = header(ii);
      break;
    }
  }

  return result;
}

http::Header* http::Message::find(const char* name)
throw() {
  Header* result = NULL;
  Header* w;
  const string* s;

  for(header_iterator ii = header_begin(), maxii = header_end(); ii != maxii; ii ++) {
    w = header(ii);

    if(w->getCategory() != Header::Category::Extension)
      continue;

    if((s = w->getExtensionName()) == NULL)
      continue;

    if(*s == name) {
      result = w;
      break;
    }
  }

  return result;
}

//-----------------------------------------------------------------------------------------------
// Codifica el mensaje HTTP.
//
// Lo unico complicado a tener en cuenta es que el valor del content-length coincida con el
// valor de la longitud real de 'body'.
//-----------------------------------------------------------------------------------------------
const DataBlock& http::Message::code()
throw(RuntimeException) {
  Header* contentLength = const_cast <Header*>(find(Header::Type::ContentLength));
  const DataBlock& body = getBody();

  if(contentLength != NULL) {
    if(body.isEmpty() == false) {
      if(contentLength->getIntegerValue() != body.getSize())
        contentLength->setValue(body.getSize());
    } else
      a_headers.release(contentLength);
  } else {
    if(body.isEmpty() == false) {
      contentLength = createHeader(Header::Type::ContentLength);
      contentLength->setValue(body.getSize());
    }
  }

  a_codeBuffer->clear();
  codeLine(codeStartLine());

  for(header_iterator ii = header_begin(), maxii = header_end(); ii != maxii; ii ++)
    codeLine(Message::header(ii)->code());

  a_codeBuffer->append(endOfLine, sizeEndOfLine);
  a_codeBuffer->append(body);
  LOGDEBUG(
    string msg("anna:http::Message::code | Message: ");
    msg += anna::functions::asString(*a_codeBuffer);
    Logger::debug(msg, ANNA_FILE_LOCATION);
  );
  return *a_codeBuffer;
}

void http::Message::codeLine(const std::string& line)
throw(RuntimeException) {
  a_codeBuffer->append(line.c_str(), line.length());
  a_codeBuffer->append(endOfLine, sizeEndOfLine);
}
