// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <time.h>
#include <ldap.h>

#include <anna/core/functions.hpp>

#include <anna/ldap/ResultCode.hpp>
#include <anna/ldap/Session.hpp>

using namespace std;

#define DoErrorCode(methodName,macroName) \
   bool anna::ldap::ResultCode::methodName () const \
      throw () \
   { \
      return a_value == (macroName); \
   }

DoErrorCode(isOperationsError, LDAP_OPERATIONS_ERROR)
DoErrorCode(isInvalidCredential, LDAP_INVALID_CREDENTIALS)
DoErrorCode(isProtocolError, LDAP_PROTOCOL_ERROR)
DoErrorCode(isTimeLimitExceeded, LDAP_TIMELIMIT_EXCEEDED)
DoErrorCode(isSizeLimitExceeded, LDAP_SIZELIMIT_EXCEEDED)
DoErrorCode(isAuthMethodNotSupported, LDAP_AUTH_METHOD_NOT_SUPPORTED)
DoErrorCode(isStrongAuthRequired, LDAP_STRONG_AUTH_REQUIRED)
DoErrorCode(isSASLBindInProgress, LDAP_SASL_BIND_IN_PROGRESS)
DoErrorCode(isTimeout, LDAP_TIMEOUT)
DoErrorCode(isUnavailable, LDAP_UNAVAILABLE)
DoErrorCode(isServerDown, LDAP_SERVER_DOWN)
DoErrorCode(isLocalError, LDAP_LOCAL_ERROR)
DoErrorCode(isDecodingError, LDAP_DECODING_ERROR)
DoErrorCode(isFilterError, LDAP_FILTER_ERROR)
DoErrorCode(isConnectError, LDAP_CONNECT_ERROR)

#define DoErrorRange(methodName,rangeName) \
   bool anna::ldap::ResultCode::methodName () const \
      throw () \
   { \
      return rangeName (a_value) != 0; \
   }

DoErrorRange(isAttrError, LDAP_ATTR_ERROR)
DoErrorRange(isNameError, LDAP_NAME_ERROR)
DoErrorRange(isSecurityError, LDAP_SECURITY_ERROR)
DoErrorRange(isServiceError, LDAP_SERVICE_ERROR)

anna::ldap::ResultCode::ResultCode(const int ldap_method_result) :
  a_value(ldap_method_result) {
  if(ldap_method_result != LDAP_SUCCESS) {
    const char* text = ldap_err2string(ldap_method_result);
    a_text = (text == NULL) ? "<undefined>" : text;
  }
}


/**
 * Al invocar a un mtodo que devuelva el resultado y adems haya que pasar una variable
 * donde contener el error pasaremos ldap::ResultCode::extraError, para que vuelve ah el
 * cdigo de error.
 + Una vez hecho eso, el cdigo de error "real" ser el mximo de los dos. Por es posible
 * que el mtodo invicado devuelva 0, pero indique el cdigo de error el extraError.
 */
anna::ldap::ResultCode& anna::ldap::ResultCode::operator= (const int ldap_method_result)
throw() {
  if((a_value = ldap_method_result) != LDAP_SUCCESS) {
    const char* text = ldap_err2string(ldap_method_result);
    a_text = (text == NULL) ? "<undefined>" : text;
  }

  return *this;
}

void anna::ldap::ResultCode::setValue(const int ldap_method_result, const int ldap_method_error)
throw() {
  if(ldap_method_result != LDAP_SUCCESS)
    operator= (ldap_method_result);
  else
    operator= (ldap_method_error);
}

const string  anna::ldap::ResultCode::asString() const
throw() {
  string result(functions::asText("ldap::ResultCode { Value: ", a_value));

  if(a_text.empty() == false) {
    result += " | Text: ";
    result += a_text;
  }

  return result += " }";
}

bool anna::ldap::ResultCode::extractResultCode(const Session* session)
throw() {
  LDAP* handle = (LDAP*)((Session*) session)->getLDAP();
  int result = -1;

  if(ldap_get_option(handle, LDAP_OPT_RESULT_CODE, &result) == LDAP_OPT_SUCCESS) {
    operator= (result);
    return true;
  }

  return false;
}
