// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <sys/types.h>

#include <anna/core/tracing/Logger.hpp>

#include <anna/timex/Engine.hpp>
#include <anna/timex/internal/TickConsumer.hpp>
#include <anna/timex/internal/TickProducer.hpp>

using namespace std;
using namespace anna;

//--------------------------------------------------------------------------------------------
// (1) Para evitar que el 'write' sobre el pipe se pueda quedar bloqueado cuando halla algn
//     problema. Se detecto el bloqueo cuando el disco sobre el que se ejecutaba la aplicacin
//     se llenaba.
// (2) Puede ser que se invoque mas de una vez.
//--------------------------------------------------------------------------------------------
void anna::timex::TickConsumer::initialize()
throw(RuntimeException) {
  if(a_pipe [0] != -1)                                        // (2)
    return;

  if(pipe(a_pipe) != 0)
    throw RuntimeException("Cannot create the pipe", errno, ANNA_FILE_LOCATION);

  if(fcntl(a_pipe [1], F_SETFL, O_NONBLOCK | O_NDELAY) == -1)    // (1)
    throw RuntimeException("Cannot establish Non-Block mode on pipe", errno, ANNA_FILE_LOCATION);

  setfd(a_pipe [0]);
}

void anna::timex::TickConsumer::apply()
throw(RuntimeException) {
  char buffer [256];

  if(a_pipe [0] == -1)
    return;

  int r;

  try {
    anna_signal_shield(r, read(a_pipe [0], buffer, sizeof(buffer)));

    if(r < 0)
      Logger::write(Logger::Critical, "Cannot recover the tick", strerror(errno), ANNA_FILE_LOCATION);

    while(r > 0) {
      a_timeController.tick();
      r --;
    }
  } catch(Exception& ex) {
    ex.trace();
  }
}

void anna::timex::TickConsumer::finalize()
throw() {
  if(a_timeController.a_tickProducer)
    a_timeController.a_tickProducer->requestStop();
}

/**
 * Si usaramos directamente el duplicado de los fd's que hace fork, todos los procesos compartirian el
 * mismo pipe, es decir, el proceso padre podria escribir al generar la alarma, pero el tratamiento del
 * tick podria hacerlo el tercer hijo, lo que haria imposible el correcto funcionamiento de las transaciones.
 *
 * Asi que cada proceso debe tener su propia copia del pipe, para que cada uno lleve independientemente
 * el control de tiempos.
 */
void anna::timex::TickConsumer::clone()
throw(RuntimeException) {
  close(a_pipe [0]);
  close(a_pipe [1]);

  if(pipe(a_pipe) != 0)
    throw RuntimeException("Cannot create the pipe", errno, ANNA_FILE_LOCATION);

  if(fcntl(a_pipe [1], F_SETFL, O_NONBLOCK | O_NDELAY) == -1)    // (1)
    throw RuntimeException("Cannot establish Non-Block mode on pipe", errno, ANNA_FILE_LOCATION);

  setfd(a_pipe [0]);

  if(a_timeController.a_tickProducer)
    a_timeController.a_tickProducer->setfd(a_pipe [1]);
}

std::string anna::timex::TickConsumer::asString() const
throw() {
  string result("timex::TickConsumer { ");
  result += Handler::asString();
  return result += functions::asString(" | Pipe (Read = %d, Write = %d) } ", a_pipe [0], a_pipe [1]);
}


