// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <anna/core/tracing/Logger.hpp>
#include <anna/core/tracing/TraceMethod.hpp>
#include <anna/config/defines.hpp>
#include <anna/core/functions.hpp>

#include <libxml/parser.h>

#include <anna/xml/DTDMemory.hpp>

using namespace std;
using namespace anna;
using namespace anna::xml;

DTDMemory::DTDMemory(const char *dtd) {
  a_filename = "/tmp/anna.xml.";
  a_filename += functions::asString((int) getpid());
  a_filename += ".dtd";
  if (dtd) initialize(dtd);
}

//---------------------------------------------------------------------------------------------
// Aunque la libXML ofrece funciones para analizar una DTD a partir de un buffer de memoria
// hemos sido incapaces de hacerla funcionar correctamente. As� que nos vemos obligados
// a volcar el buffer en un archivo .. y una vez all� analizarlo con la funci�n que analiza
// la DTD a partir de un archivo.
//---------------------------------------------------------------------------------------------
_xmlDtd* DTDMemory::parse(const char* content) const
throw(RuntimeException) {
  LOGMETHOD(TraceMethod tf("anna::xml::DTDMemory", "parse", ANNA_FILE_LOCATION));
  _xmlDtd* result;
  int stream;
  LOGDEBUG(
    Logger::write(Logger::Debug, "DTD", content, ANNA_FILE_LOCATION);
    Logger::write(Logger::Debug, "Temporary file", a_filename.c_str(), ANNA_FILE_LOCATION)
  );

  if((stream = open(a_filename.c_str(), O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR)) == -1)
    throw RuntimeException(a_filename, errno, ANNA_FILE_LOCATION);

  if(write(stream, content, anna_strlen(content)) == -1) {
    const int xerrno = errno;
    close(stream);
    unlink(a_filename.c_str());
    throw RuntimeException(a_filename, xerrno, ANNA_FILE_LOCATION);
  }

  close(stream);

  if((result = xmlParseDTD(NULL, BAD_CAST(a_filename.c_str()))) == NULL) {
    unlink(a_filename.c_str());
    throw RuntimeException("DTD is wrong", ANNA_FILE_LOCATION);
  }

  unlink(a_filename.c_str());
  return result;
}

